@@ -22,7 +22,7 @@ use Unicode::Normalize;
 use parent qw(Class::Accessor);
 __PACKAGE__->follow_best_practice;
 
-our $VERSION = '2.17';
+our $VERSION = '2.18';
 our $BETA_VERSION = 0; # Is this a beta version?
 
 our $logger  = Log::Log4perl::get_logger('main');
@@ -72,6 +72,7 @@ $CONFIG->{state}{graph} = {};
 # Track the order of keys as cited. Keys cited in the same \cite*{} get the same order
 # Used for sorting schemes which use \citeorder
 $CONFIG->{state}{keyorder} = {};
+$CONFIG->{state}{internalkeyorder} = {};
 
 # Location of the control file
 $CONFIG->{state}{control_file_location} = '';
@@ -223,9 +224,7 @@ sub _initopts {
 
   # cache meta markers since they are referenced in the oft-called _get_handler
   $CONFIG_META_MARKERS{annotation} = quotemeta(Biber::Config->getoption('annotation_marker'));
-
   $CONFIG_META_MARKERS{namedannotation} = quotemeta(Biber::Config->getoption('named_annotation_marker'));
-  $CONFIG_META_MARKERS{xname} = quotemeta(Biber::Config->getoption('xname_marker'));
 
   # Setting up Log::Log4perl
   my $LOGLEVEL;
@@ -302,7 +301,7 @@ sub _initopts {
 
   my $vn = $VERSION;
   $vn .= ' (beta)' if $BETA_VERSION;
-  my $tool = ' running in TOOL mode' if Biber::Config->getoption('tool');
+  my $tool = Biber::Config->getoption('tool') ? ' running in TOOL mode' : '';
 
   $logger->info("This is Biber $vn$tool") unless Biber::Config->getoption('nolog');
 
@@ -489,7 +488,8 @@ sub _config_file_set {
           }
           push $snkps->@*, $snps;
         }
-        $snss->{$sns->{name}} = $snkps;
+        $snss->{$sns->{name}}{visibility} = $sns->{visibility};
+        $snss->{$sns->{name}}{template} = $snkps;
       }
       Biber::Config->setblxoption(0, 'sortingnamekeytemplate', $snss);
     }
@@ -926,6 +926,10 @@ sub addtoblxoption {
 sub setblxoption {
   shift; # class method so don't care about class name
   my ($secnum, $opt, $val, $scope, $scopeval) = @_;
+
+  # Map booleans to 1 and 0 for consistent testing
+  $val = Biber::Utils::map_boolean($opt, $val, 'tonum');
+
   if (not defined($scope)) { # global is the default
     if ($CONFIG_OPTSCOPE_BIBLATEX{$opt}{GLOBAL}) {
       $CONFIG->{options}{biblatex}{GLOBAL}{$opt} = $val;
@@ -964,6 +968,11 @@ sub getblxoption {
   no autovivification;
   shift; # class method so don't care about class name
   my ($secnum, $opt, $entrytype, $citekey) = @_;
+  # Set impossible defaults
+  $secnum //= "\x{10FFFD}";
+  $opt //= "\x{10FFFD}";
+  $entrytype //= "\x{10FFFD}";
+  $citekey //= "\x{10FFFD}";
   if ( defined($citekey) and
        $CONFIG_OPTSCOPE_BIBLATEX{$opt}{ENTRY} and
        defined $CONFIG->{options}{biblatex}{ENTRY}{$citekey}{$secnum}{$opt}) {
@@ -1114,7 +1123,7 @@ sub is_inheritance_path {
 
 =head2 set_keyorder
 
-  Set some key order information
+  Set key order information
 
 =cut
 
@@ -1125,9 +1134,22 @@ sub set_keyorder {
   return;
 }
 
+=head2 set_internal_keyorder
+
+  Set key order information for keys with the same order
+
+=cut
+
+sub set_internal_keyorder {
+  shift; # class method so don't care about class name
+  my ($section, $key, $intkeyorder) = @_;
+  $CONFIG->{state}{internalkeyorder}{$section}{$key} = $intkeyorder;
+  return;
+}
+
 =head2 get_keyorder
 
-  Get some key order information
+  Get key order information
 
 =cut
 
@@ -1137,6 +1159,18 @@ sub get_keyorder {
   return $CONFIG->{state}{keyorder}{$section}{$key};
 }
 
+=head2 get_internal_keyorder
+
+  Get key order information for keys with the same order
+
+=cut
+
+sub get_internal_keyorder {
+  shift; # class method so don't care about class name
+  my ($section, $key) = @_;
+  return $CONFIG->{state}{internalkeyorder}{$section}{$key};
+}
+
 
 =head2 get_keyorder_max
 
