@@ -41,13 +41,14 @@ our @EXPORT = qw{
                   %MONTHS
                   %RSTRINGS
                   %USEDSTRINGS
+                  %YEARDIVISIONS
               };
 
 # Version of biblatex control file which this release expects. Matched against version
 # passed in control file. Used when checking the .bcf
-our $BCF_VERSION = '3.7';
+our $BCF_VERSION = '3.9';
 # Format version of the .bbl. Used when writing the .bbl
-our $BBL_VERSION = '3.1';
+our $BBL_VERSION = '3.2';
 
 # Global flags needed for sorting
 our $BIBER_SORT_FINAL;
@@ -70,6 +71,29 @@ unless ($locale) {
   }
 }
 
+# ISO8601-2 4.8 year divisions
+our %YEARDIVISIONS = ( 21 => 'spring',
+                       22 => 'summer',
+                       23 => 'autumn',
+                       24 => 'winter',
+                       25 => 'springN',
+                       26 => 'summerN',
+                       27 => 'autumnN',
+                       28 => 'winterN',
+                       29 => 'springS',
+                       30 => 'summerS',
+                       31 => 'autumnS',
+                       32 => 'winterS',
+                       33 => 'Q1',
+                       34 => 'Q2',
+                       35 => 'Q3',
+                       36 => 'Q4',
+                       37 => 'QD1',
+                       38 => 'QD2',
+                       39 => 'QD3',
+                       40 => 'S1',
+                       41 => 'S2' );
+
 # Reverse record of macros so we can reverse these for tool mode output
 our %RSTRINGS = ();
 # Record of macros which are actually used in output in tool mode, so that we don't
@@ -128,9 +152,12 @@ our %DM_DATATYPES = (
                            return 0;
                          }
                        }
-                       elsif ($f =~ /season$/) {
+                       elsif ($f =~ /season$/) { # LEGACY
                          return 0 unless $v =~ m/(?:winter|spring|summer|autumn)/
                        }
+                       elsif ($f =~ /yeardivision$/) {
+                         return 0 unless grep {$v eq $_} values %YEARDIVISIONS;
+                       }
                        else {
                          # num() doesn't like negatives
                          return 0 unless looks_like_number(num($v =~ s/^-//r));
@@ -233,6 +260,7 @@ our $CONFIG_DEFAULT_BIBER = {
   nolabel                                     => { option => [ {value => q/[\p{Pc}\p{Ps}\p{Pe}\p{Pi}\p{Pf}\p{Po}\p{S}\p{C}]+/} ] },
 #  nolabelwidthcount                          => { option =>  }, # default is nothing
   nolog                                       => { content => 0 },
+#  nonamestring                               => { option =>  }, # default is nothing
   noskipduplicates                            => { content => 0 },
   nostdmacros                                 => { content => 0 },
   nosort                                      => { option => [ { name => 'setnames', value => q/\A\p{L}{2}\p{Pd}(?=\S)/ },
@@ -652,7 +680,7 @@ L<https://github.com/plk/biber/issues>.
 =head1 COPYRIGHT & LICENSE
 
 Copyright 2009-2012 François Charette and Philip Kime, all rights reserved.
-Copyright 2012-2020 Philip Kime, all rights reserved.
+Copyright 2012-2022 Philip Kime, all rights reserved.
 
 This module is free software.  You can redistribute it and/or
 modify it under the terms of the Artistic License 2.0.
