@@ -45,7 +45,7 @@ sub _getnamehash {
   my $bee = $be->get_field('entrytype');
 
   my $hashkey = '';
-  my $count = $names->count_names;
+  my $count = $names->count;
   my $visible = $bib ? $dlist->get_visible_bib($names->get_id) : $dlist->get_visible_cite($names->get_id);
   my $dm = Biber::Config->get_dm;
   my @nps = $dm->get_constant_value('nameparts');
@@ -86,7 +86,7 @@ sub _getfullhash {
   foreach my $n ($names->names->@*) {
     foreach my $nt (@nps) {# list type so returns list
       if (my $np = $n->get_namepart($nt)) {
-        $hashkey .= $np;
+        $hashkey .= strip_nonamestring($np, $names->get_type);
       }
     }
   }
@@ -110,7 +110,7 @@ sub _getnamehash_u {
   my $bee = $be->get_field('entrytype');
 
   my $hashkey = '';
-  my $count = $names->count_names;
+  my $count = $names->count;
   my $nlid = $names->get_id;
   my $visible = $dlist->get_visible_cite($nlid);
   my $dm = Biber::Config->get_dm;
@@ -284,7 +284,7 @@ sub _labelpart {
       }
       if ( first {$f eq $_} $dm->get_fields_of_type('list', 'name')->@*) {
         my $name = $be->get_field($f) || next; # just in case there is no labelname etc.
-        my $total_names = $name->count_names;
+        my $total_names = $name->count;
         my $visible_names;
         if ($total_names > $maxan) {
           $visible_names = $minan;
@@ -446,7 +446,7 @@ sub _label_name {
       $useprefix = $names->get_useprefix;
     }
 
-    my $numnames  = $names->count_names;
+    my $numnames  = $names->count;
     my $visibility = $dlist->get_visible_alpha($names->get_id);
 
     # Use name range override, if any
@@ -1009,6 +1009,8 @@ my %internal_dispatch_sorting = (
                                  'editorbtype'     =>  [\&_sort_editort,       ['editorbtype']],
                                  'editorctype'     =>  [\&_sort_editort,       ['editorctype']],
                                  'citeorder'       =>  [\&_sort_citeorder,     []],
+                                 'citecount'       =>  [\&_sort_citecount,     []],
+                                 'intciteorder'    =>  [\&_sort_intciteorder,  []],
                                  'labelalpha'      =>  [\&_sort_labelalpha,    []],
                                  'labelname'       =>  [\&_sort_labelname,     []],
                                  'labeltitle'      =>  [\&_sort_labeltitle,    []],
@@ -1100,7 +1102,7 @@ sub _dispatch_sorting {
     $code_args_ref  = $d->[1];
   }
   else { # Unknown field
-    biber_warn("Unknown field '$sortfield' found in sorting template");
+    biber_warn("Field '$sortfield' in sorting template is not a sortable field");
     return undef;
   }
 
@@ -1203,15 +1205,22 @@ sub _sort_citeorder {
   my ($self, $citekey, $secnum, $section, $be, $dlist, $sortelementattributes) = @_;
   # Allkeys and sorting=none means use bib order which is in orig_order_citekeys
   # However, someone might do:
-  # \cite{b,a}
+  # \cite{b}\cite{a}
   # \nocite{*}
   # in the same section which means we need to use the order attribute for those
   # keys which have one (the \cited keys) and then an orig_order_citekey index based index
   # for the nocite ones.
   my $ko = Biber::Config->get_keyorder($secnum, $citekey);# only for \cited keys
   if ($section->is_allkeys) {
-    return $ko || (Biber::Config->get_keyorder_max($secnum) +
-                   (first_index {$_ eq $citekey} $section->get_orig_order_citekeys) + 1);
+    my $biborder = (Biber::Config->get_keyorder_max($secnum) +
+                    (first_index {$_ eq $citekey} $section->get_orig_order_citekeys) + 1);
+    my $allkeysorder = Biber::Config->get_keyorder($secnum, '*');
+    if (defined($ko) and defined($allkeysorder) and $allkeysorder < $ko) {
+      return $biborder;
+    }
+    else {
+      return $ko || $biborder;
+    }
   }
   # otherwise, we need to take account of citations with simulataneous order like
   # \cite{key1, key2} so this tied sorting order can be further sorted with other fields
@@ -1222,6 +1231,11 @@ sub _sort_citeorder {
   }
 }
 
+sub _sort_citecount {
+  my ($self, $citekey, $secnum, $section, $be, $dlist, $sortelementattributes) = @_;
+  return $section->get_citecount($citekey) // '';
+}
+
 sub _sort_integer {
   my ($self, $citekey, $secnum, $section, $be, $dlist, $sortelementattributes, $args) = @_;
   my $dmtype = $args->[0]; # get int field type
@@ -1235,7 +1249,7 @@ sub _sort_integer {
     }
 
     # Use Unicode::UCD::num() to map Unicode numbers to integers if possible
-    $field = num($field) //$field;
+    $field = num($field) // $field;
 
     return _process_sort_attributes($field, $sortelementattributes);
   }
@@ -1267,6 +1281,11 @@ sub _sort_entrytype {
   return _process_sort_attributes($be->get_field('entrytype'), $sortelementattributes);
 }
 
+sub _sort_intciteorder {
+  my ($self, $citekey, $secnum, $section, $be, $dlist, $sortelementattributes) = @_;
+  return Biber::Config->get_internal_keyorder($secnum, $citekey);
+}
+
 sub _sort_labelalpha {
   my ($self, $citekey, $secnum, $section, $be, $dlist, $sortelementattributes, $args) = @_;
   my $string = $dlist->get_entryfield($citekey, 'sortlabelalpha') // '';
@@ -1476,9 +1495,7 @@ sub _namestring {
   my $bee = $be->get_field('entrytype');
   my $names = $be->get_field($field);
   my $str = '';
-  my $count = $names->count_names;
-  # get visibility for sorting
-  my $visible = $dlist->get_visible_sort($names->get_id);
+  my $count = $names->count;
   my $useprefix = Biber::Config->getblxoption($secnum, 'useprefix', $bee, $citekey);
 
   # Get the sorting name key template for this list context
@@ -1490,6 +1507,20 @@ sub _namestring {
   # Override with any namelist scope sorting name key template option
   $snkname = $names->get_sortingnamekeytemplatename // $snkname;
 
+  # Get the sorting namekey template determined so far now that we are down to the name list
+  # scope since we need the visibility type now and this doesn't mean anything below the name list
+  # level anyway. We will select the final sorting namekey template below if there is an override
+  # at the individual name level
+  my $tmpsnk = Biber::Config->getblxoption(undef, 'sortingnamekeytemplate')->{$snkname};
+  # Now set visibility of the correct type. By default this is the standard
+  # sorting visibility but can be citation visibility as the biblatex
+  # "sortcites" option can require a different visibility for citations and
+  # so we have to generate a separate sorting list for this case
+  my $visible = $dlist->get_visible_sort($names->get_id);
+  if (defined($tmpsnk) and $tmpsnk->{visibility} eq 'cite') {
+    $visible = $dlist->get_visible_cite($names->get_id);
+  }
+
   # Name list scope useprefix option
   if (defined($names->get_useprefix)) {
     $useprefix = $names->get_useprefix;
@@ -1497,11 +1528,6 @@ sub _namestring {
 
   my $trunc = "\x{10FFFD}";  # sort string for "et al" truncated name
 
-  # We strip nosort first otherwise normalise_string_sort damages diacritics
-  # We strip each individual component instead of the whole thing so we can use
-  # as name separators things which would otherwise be stripped. This way we
-  # guarantee that the separators are never in names
-
   foreach my $n ($names->first_n_names($visible)->@*) {
 
     # Name scope useprefix option
@@ -1510,6 +1536,9 @@ sub _namestring {
     }
 
     # Override with any name scope sorting name key template option
+    # This won't override the visibility type selection already taken from higher-level
+    # sorting namekey templates since this option only applies at name list level and higher
+    # anyway and this is individual name scope
     $snkname = $n->get_sortingnamekeytemplatename // $snkname;
 
     # Now get the actual sorting name key template
@@ -1517,7 +1546,7 @@ sub _namestring {
 
     # Get the sorting name key specification and use it to construct a sorting key for each name
     my $kpa = [];
-    foreach my $kp ($snk->@*) {
+    foreach my $kp ($snk->{template}->@*) {
       my $kps = '';
       for (my $i=0; $i<=$kp->$#*; $i++) {
         my $np = $kp->[$i];
@@ -1620,6 +1649,10 @@ sub _liststring {
   }
 
   # separate the items by a string to give some structure
+  # We strip nosort first otherwise normalise_string_sort damages diacritics
+  # We strip each individual component instead of the whole thing so we can use
+  # as name separators things which would otherwise be stripped. This way we
+  # guarantee that the separators are never in names
   if ($verbatim) { # no normalisation for verbatim/uri fields
     $str = join($lsi, map { strip_nosort($_, $field)} @items);
   }
@@ -1671,7 +1704,7 @@ L<https://github.com/plk/biber/issues>.
 =head1 COPYRIGHT & LICENSE
 
 Copyright 2009-2012 François Charette and Philip Kime, all rights reserved.
-Copyright 2012-2020 Philip Kime, all rights reserved.
+Copyright 2012-2022 Philip Kime, all rights reserved.
 
 This module is free software.  You can redistribute it and/or
 modify it under the terms of the Artistic License 2.0.
